@@ -165,11 +165,11 @@ a23,
 a24
 )
 {
-if(this._isConvertible())
+a23=TrFormatUtils.trim(a23);
+if(this._isConvertible(a23))
 {
 if(a23==null)
 return null;
-a23=TrUIUtils.trim(a23);
 if(a23.length==0)
 return null
 var a25;
@@ -237,25 +237,27 @@ else
 return undefined;
 }
 }
-TrNumberConverter.prototype._isConvertible=function()
+TrNumberConverter.prototype._isConvertible=function(a36)
 {
-return(this._pattern==null);
+if(this._pattern!=null)
+return false;
+return TrFormatUtils.isNumberConvertible(a36);
 }
-TrNumberConverter.prototype._initNumberFormat=function(a36)
+TrNumberConverter.prototype._initNumberFormat=function(a37)
 {
 if(this._type=="percent")
 {
 this._example=0.3423;
-this._numberFormat=TrNumberFormat.getPercentInstance(a36);
+this._numberFormat=TrNumberFormat.getPercentInstance(a37);
 }
 else if(this._type=="currency")
 {
 this._example=10250;
-this._numberFormat=TrNumberFormat.getCurrencyInstance(a36);
+this._numberFormat=TrNumberFormat.getCurrencyInstance(a37);
 }
 else if(this._type=="number")
 {
-this._numberFormat=TrNumberFormat.getNumberInstance(a36);
+this._numberFormat=TrNumberFormat.getNumberInstance(a37);
 }
 this._numberFormat.setGroupingUsed(this.isGroupingUsed());
 this._numberFormat.setMaximumFractionDigits(this.getMaxFractionDigits());
